/**
 * ColorSwap module.
 * Assists in the swapping of RGB colour template images.
 */

var ColorSwap = {
	/**
	 * Swap the R, G and B components of a surface
	 * with other colours.
	 */
	swapSurface: function (surface, redColor, greenColor, blueColor) {
		var m = CreateColorMatrix(
				0, redColor.red, greenColor.red, blueColor.red,
				0, redColor.green, greenColor.green, blueColor.green,
				0, redColor.blue, greenColor.blue, blueColor.blue);
		surface.applyColorFX(0, 0, surface.width, surface.height, m);
		return surface;
	},
	
	/**
	 * Swaps the R, G and B components of an image
	 * with other colours.
	 */
	swapImage: function (image, redColor, greenColor, blueColor) {
		return ColorSwap.swapSurface(image.createSurface(),
				redColor, greenColor, blueColor).createImage();
	},
	
	/**
	 * Swaps the R, G and B components of a full spriteset
	 * with other colours.
	 */
	swapSpriteset: function (spriteset, redColor, greenColor, blueColor) {
		for (var i = 0; i < spriteset.images.length; ++i) {
			spriteset.images[i] = ColorSwap.swapImage(spriteset.images[i],
					redColor, greenColor, blueColor);
		}
		return spriteset;
	}
};