/**
 * Items module.
 * Handles the display and use of items.
 */

var Items = {
	/**
	 * Holds item function handles.
	 */
	uses: [],
	itemUsed: false,
	
	/**
	 * Healing item template.
	 */
	healItem: function (stat, amount) {
		return function (battle) {
			hero[stat] += amount;
			if (hero[stat] > hero[stat + "Max"]) {
				hero[stat] = hero[stat + "Max"];
			}
			
			return true;
		};
	},
	addHealItem: function (name, stat, amount) {
		Items.uses[name] = Items.healItem(stat, amount);
	},
	
	/**
	 * Equippable item template.
	 */
	switchEquip: function (name, type, stat, strength) {
		return function (battle) {
			// Don't allow switching in battle.
			if (battle) {
				TextBox.say("You can't switch equipment in combat!");
				return false;
			}
			
			// Switch out the old.
			var old = hero[type];
			hero[type] = "";
			hero[stat] -= hero[type + "Level"];
			inventory.push(old);
			
			// Bring in the new!
			hero[stat] += strength;
			hero[type] = name;
			hero[type + "Level"] = strength;
			
			return true;
		};
	},
	addWeapon: function (name, strength) {
		Items.uses[name] = Items.switchEquip(name, "weapon", "str", strength);
	},
	addArmor: function (name, defense) {
		Items.uses[name] = Items.switchEquip(name, "armor", "def", defense);
	},
	
	/**
	 * Summons the items menu.
	 * Items can be used or equipped.
	 */
	menu: function (x, y, preRender, battle) {
		// Defaults
		if (x === undefined) x = 20;
		if (y === undefined) y = 20;
		if (preRender === undefined) preRender = function () {
			if (IsMapEngineRunning()) RenderMap();
			drawHeroStats();
		};
		if (battle === undefined) battle = false;
		
		var menu = new DefaultMenu(x, y, 100, 100);
		menu.mouseSupport = false;
		Items.itemUsed = false;
		
		// Items menu drawing
		var menuPreRender = menu.preRender;
		menu.preRender = function () {
			preRender();
			menuPreRender.apply(menu);
		};
		
		// Add all items.
		Func.forEachIndex(inventory, function (index, item) {
			menu.addText(item, function () {
				if (Util.contains(Util.keys(Items.uses), item)) {
					if (Items.uses[item](battle)) {
						inventory.splice(index, 1);
						menu.done = true;
						Items.itemUsed = true;
					}
				}
			});
		});
		
		// Add obligatory exit command.
		menu.addText("Back", function () {
			menu.done = true;
		});
		
		menu.execute();
	}
};

/** Items! */

Items.addHealItem("Potion", "hp", 10);
Items.addHealItem("S. Potion", "hp", 50);
Items.addHealItem("M. Potion", "hp", Number.MAX_VALUE);

Items.addHealItem("Ether", "mp", 10);
Items.addHealItem("S. Ether", "mp", 50);
Items.addHealItem("M. Ether", "mp", Number.MAX_VALUE);

Items.uses["Elixir"] = function () {
	Items.uses["M. Potion"]();
	Items.uses["M. Ether"]();
	return true;
};

/** Weapons and armour */

Items.addWeapon("wut sword", 1);
Items.addWeapon("Stick", 5);
Items.addWeapon("Knife", 10);
Items.addWeapon("Sword", 20);
Items.addWeapon("Super sword", 50);

Items.addArmor("wut armour", 1);
Items.addArmor("Shirt", 5);
Items.addArmor("Jacket", 10);
Items.addArmor("Vest", 20);
Items.addArmor("Force field", 50);