/**
 * Magic module.
 * Holds magic logic plus spell definitions.
 */

var Magic = {
	/**
	 * Holds spell uses.
	 */
	uses: [],
	magicUsed: false,
	
	/**
	 * Combat spell template.
	 */
	combatSpell: function (name, func) {
		return function (battle) {
			if (!battle) {
				TextBox.say("You can't cast " + name + " outside of combat!");
				return false;
			}
			// Cast the spell.
			func();
			return true;
		};
	},
	addCombatSpell: function (name, func) {
		Magic.uses[name] = Magic.combatSpell(name, func);
	},
	
	/**
	 * Healing spell template
	 */
	healingSpell: function (name) {
		return function (battle) {
			var oldHp = hero.hp;
			var spellNames = Func.map(function (s) { return s.name; }, spells);
			hero.hp += spells[Util.indexOf(spellNames, name)].level * 10;
			if (hero.hp > hero.hpMax) {
				hero.hp = hero.hpMax;
			}
			
			var healAmount = hero.hp - oldHp;
			if (healAmount > 0) {
				TextBox.say("Casted " + name + "! " + (hero.hp - oldHp) +
						" HP restored!");
				return true;
			} else {
				TextBox.say("Already at full health.");
				return false;
			}
		};
	},
	addHealingSpell: function (name) {
		Magic.uses[name] = Magic.healingSpell(name);
	},
	
	/**
	 * Standard spell cost.
	 */
	spellCost: function (level) {
		return level * 3;
	},
	
	/**
	 * Summons the magic menu.
	 */
	menu: function (x, y, preRender, battle) {
		// Defaults
		if (x === undefined) x = 20;
		if (y === undefined) y = 20;
		if (preRender === undefined) preRender = function () {
			if (IsMapEngineRunning()) RenderMap();
			drawHeroStats();
		};
		if (battle === undefined) battle = false;
		
		var menu = new DefaultMapMenu(x, y, 120, 100);
		menu.mouseSupport = false;
		Magic.magicUsed = false;
		
		// Magic menu drawing
		var menuPreRender = menu.preRender;
		menu.preRender = function () {
			preRender();
			menuPreRender.apply(menu);
		};
		
		// Add all spells.
		Func.forEach(spells, function (spell) {
			if (spell.level > 0) {
				menu.addText(
					Magic.spellCost(spell.level) + "MP " + spell.name + spell.level,
					function () {
						if (hero.mp < Magic.spellCost(spell.level)){
							TextBox.say("Not enough MP to cast " + spell.name + "!");
						} else if (Magic.uses[spell.name](battle)) {
							hero.mp -= Magic.spellCost(spell.level);
							Magic.magicUsed = true;
							menu.done = true;
						}
					}
				);
			}
		});
		menu.addText("Back", function () {
			menu.done = true;
		});
		
		menu.execute();
	}
};

/** Spells */

Magic.addCombatSpell("Earth", function () {});
Magic.addCombatSpell("Fire", function () {});
Magic.addCombatSpell("Wind", function () {});
Magic.addCombatSpell("Water", function () {});

Magic.addHealingSpell("Heart");