/**
 * Person module.
 * Just about everything mundane about handling persons.
 */

var Person = {	
	/**
	 * Check if a person exists.
	 */
	exists: function (person) {
		return Util.indexOf(GetPersonList(), person) !== -1;
	},
	
	/**
	 * Sets an initial direction for a person.
	 */
	initialFace: function (person, direction) {
		QueuePersonScript(person, 'SetPersonDirection("' + person + '", "' +
				direction + '");', false);
	},
	
	/**
	 * Substitute the colours of a person's spriteset.
	 */
	colorSwap: function (person, redColor, greenColor, blueColor) {
		var ss = GetPersonSpriteset(person);
		ss = ColorSwap.swapSpriteset(ss, redColor, greenColor, blueColor);
		SetPersonSpriteset(person, ss);
	},
	
	/**
	 * Color and face a direction conveniently.
	 */
	colorAndFace: function (person, dir, rr, rg, rb, gr, gg, gb, br, bg, bb) {
		Person.colorSwap(person,
				CreateColor(rr, rg, rb),
				CreateColor(gr, gg, gb),
				CreateColor(br, bg, bb));
		Person.initialFace(person, dir);
	},
	
	/**
	 * Makes the hero person if they don't exist yet.
	 */
	makeHero: function () {
		if (!Person.exists("Hero")) {
			CreatePerson("Hero", "male.rss", false);
			SetPersonFrameRevert("Hero", 1);
			Person.colorAndFace("Hero", "south",
					96, 64, 0,
					224, 192, 128,
					0, 0, 255);
		}
	},
	
	/**
	 * Warp to a different map and location.
	 */
	warp: function (person, tx, ty, map, dir) {
		if (map !== undefined)
			ChangeMap(map);
		if (tx !== undefined)
			SetPersonX(person, tx * 16 + 8);
		if (ty !== undefined)
			SetPersonY(person, ty * 16 + 8);
		if (dir !== undefined)
			SetPersonDirection(person, dir);
	},
	
	/**
	 * Create a map person chest containing an item.
	 */
	makeChest: function (person, itemName) {
		var opened = false;
		
		function save() {
			return {
				opened: opened
			};
		}
		
		function load(state) {
			opened = state.opened;
		}
		
		function create() {
			Person.initialFace(person, opened ? "open" : "closed");
		}
		
		function talk() {
			if (opened) {
				TextBox.say("Empty.");
			} else {
				inventory.push(itemName);
				TextBox.say("Found " + itemName + "! Booyah!");
				opened = true;
				SetPersonDirection(person, "open");
			}
		}
		
		return {
			save: save,
			load: load,
			
			create: create,
			talk: talk
		};
	}
};