/**
 * TextBox module
 * Shows text in boxes.
 */

RequireScript("Utility.js");

var TextBox = function () {
	var font = GetSystemFont();
	var windowStyle = GetSystemWindowStyle();
	
	// For showLocation.
	var locationDuration = 0;
	var locationName = "";
	
	function drawTextBox(text, preRender) {
		var w = GetScreenWidth() / 2;
		var h = font.getHeight() * 3;
		var x = (GetScreenWidth() - w) / 2;
		var y = GetScreenHeight() - (h + font.getHeight() * 2);
		
		if (preRender === undefined) preRender = function () {
			if (IsMapEngineRunning()) {
				RenderMap();
			}
		};
		
		preRender();
		
		windowStyle.drawWindow(x, y, w, h);
		font.drawTextBox(x, y, w, h, 0, text);
	}
	
	return {
		/**
		 * Show the name of the location.
		 * Mostly for use in a render script.
		 * duration = clock ticks before hiding the display.
		 */
		showLocation: function (name, duration) {
			if (name !== locationName) {
				// New place, recalculate display.
				locationName = name;
				locationDuration = duration;
			}
			
			if (locationDuration > 0 && name !== "") {
				// Show the display.
				var w = font.getStringWidth(locationName);
				var h = font.getHeight();
				var x = (GetScreenWidth() - w) / 2;
				var y = font.getHeight() * 2;
				
				windowStyle.drawWindow(x, y, w, h);
				font.drawText(x, y, locationName);
				--locationDuration;
			} else {
				SetRenderScript('');
			}
		},
		
		/**
		 * Convenience function for showLocation.
		 */
		renderLocation: function (location) {
			SetRenderScript('TextBox.showLocation("' + location + '", 120);');
		},
		
		/**
		 * Clear the location box.
		 */
		clearLocation: function () {
			locationName = "";
		},
		
		/**
		 * Display a stock-standard text speech box.
		 */
		say: function (text, preRender) {
			drawTextBox(text, preRender);
			FlipScreen();
			Util.flushKeyBuffer();
			GetKey();
		},
		
		/**
		 * Like say, but without FlipScreen() or GetKey().
		 */
		drawSay: function (text, preRender) {
			drawTextBox(text, preRender);
		},
	};
}();