/**
 * Utility module (Util, Func)
 * Filled with useful things.
 */

var Func = {
	forEach: function (arr, action) {
		for (var i = 0; i < arr.length; ++i) {
			action(arr[i]);
		}
	},
	
	forEachIndex: function (arr, action) {
		var index = 0;
		Func.forEach(arr, function (element) {
			action(index, element);
			++index;
		});
	},
	
	map: function (transform, arr) {
		var result = [];
		Func.forEach(arr, function (element) {
			result.push(transform(element));
		});
		return result;
	},
	
	filter: function (predicate, arr) {
		var result = [];
		Func.forEach(arr, function (element) {
			if (predicate(element)) {
				result.push(element);
			}
		});
		return result;
	},
	
	reduce: function (combine, base, arr) {
		Func.forEach(arr, function (element) {
			base = combine(base, element);
		});
		return base;
	},
	
	getProperty: function (property) {
		return function (object) {
			return object[property];
		};
	}
};

var Util = {
	/**
	 * Check if an element is in an array.
	 */
	indexOf: function (arr, element) {
		var index = -1;
		return Func.reduce(function (c, n) {
			++index;
			if (c === -1 && n === element) {
				c = index;
			}
			return c;
		}, -1, arr);
	},
	
	/**
	 * Checks if an array contains an element.
	 */
	contains: function (arr, element) {
		return Util.indexOf(arr, element) !== -1;
	},
	
	/**
	 * Flush the key buffer.
	 */
	flushKeyBuffer: function () {
		while (AreKeysLeft()) {
			GetKey();
		}
	},
	
	/**
	 * Repeat a command a set number of times.
	 */
	repeat: function (times, func) {
		for (var i = 0; i < times; ++i) {
			func();
		}
	},
	
	/**
	 * Get all the members of an object in the form of an array.
	 */
	keys: function (object) {
		var result = [];
		for (var p in object) if (Object.hasOwnProperty.call(object, p)) {
			result.push(p);
		}
		return result;
	},
	
	/**
	 * Check if a file exists in a directory.
	 */
	fileExists: function (dir, file) {
		return Util.contains(GetFileList(dir), file);
	},
	
	/**
	 * Grabs an image from a spriteset.
	 */
	getSpritesetImage: function (ss, dir, frame) {
		var dirIndex = Util.indexOf(Func.map(Func.getProperty("name"), ss.directions), dir);
		var imageIndex = ss.directions[dirIndex].frames[frame].index;
		return ss.images[imageIndex];
	}
};