/**
 * SlobTown.
 * Starting town for the game.
 */

/** STATE */

var firstTime = true;

function save() {
	return {
		firstTime: firstTime
	};
}

function load(state) {
	firstTime = state.firstTime;
}

/** MAP EVENTS */

function mapEnter() {
	if (firstTime) {
		UpdateMapEngine();
		TextBox.say("The princess has been kidnapped by a dragon.");
		TextBox.say("Are you a bad enough dude to save the princess?");
		firstTime = false;
	}
	
	TextBox.renderLocation("Town of Slob");
}

function mapLeaveSouth() {
	Person.warp("Hero", 6, 17, "World.rmp");
}

/** PERSONS */

// King
function King() {
	function create() {
		Person.colorAndFace("King", "south",
			255, 0, 0,
			192, 164, 128,
			255, 255, 0);
	}
	
	function talk() {
		TextBox.say("I'm the king. Save my daughter.");
	}
	
	function generator() {
		QueuePersonCommand("King", COMMAND_ANIMATE, false);
	}
	
	return {
		create: create,
		talk: talk,
		generator: generator
	};
}

// StoreGuy
function StoreGuy() {
	function create() {
		Person.colorAndFace("StoreGuy", "west",
			192, 192, 64,
			192, 164, 128,
			64, 64, 128);
	}
	
	function talk() {
		TextBox.say("I've got nothin' to sell, so beat it!");
		
		var items = Func.filter(function (i) {
			return i.indexOf("Potion") !== -1 || i.indexOf("Ether") !== -1;
		}, Util.keys(Items.uses));
		var item = items[Math.floor(Math.random() * items.length)];
		inventory.push(item);
		TextBox.say("Received " + item + "!");
	}
	
	return {
		create: create,
		talk: talk
	};
}

// InnGuy
function InnGuy() {
	function create() {
		Person.colorAndFace("InnGuy", "south",
			32, 32, 32,
			192, 164, 128,
			128, 128, 128);
	}
	
	function talk() {
		TextBox.say("What's that, you want to sleep?");
		hero.hp = hero.hpMax;
		hero.mp = hero.mpMax;
		TextBox.say("There. Hope you had a good night's rest.");
	}
	
	return {
		create: create,
		talk: talk
	};
}

// Spinner
function Spinner() {
	function create() {
		Person.colorAndFace("Spinner", "east",
			255, 255, 128,
			192, 164, 128,
			192, 64, 32);
	}
	
	function generator() {
		var cmds = [];
		function twirl(dir) {
			Util.repeat(12, function () {
				cmds.push(COMMAND_ANIMATE);
				cmds.push(COMMAND_WAIT);
			});
			cmds.push(dir)
		}
		twirl(COMMAND_FACE_SOUTH);
		twirl(COMMAND_FACE_WEST);
		twirl(COMMAND_FACE_NORTH);
		twirl(COMMAND_FACE_EAST);
		
		Func.forEach(cmds, function (e) {
			QueuePersonCommand("Spinner", e, false);
		});
	}
	
	return {
		create: create,
		generator: generator
	};
}

// male_1
function male_1() {
	function create() {
		Person.colorAndFace("male_1", "south",
			255, 128, 255,
			192, 164, 128,
			32, 64, 32);
	}
	
	function talk() {
		TextBox.say("The dragon lives in a tower in the north.");
	}
	
	return {
		create: create,
		talk: talk
	};
}

// Tunginobi
function Tunginobi() {
	function create() {
		Person.colorAndFace("Tunginobi", "north",
			0, 0, 0,
			192, 164, 128,
			128, 128, 128
		);
	}
	
	function talk() {
		TextBox.say("This game sucks. I should know, I made it.");
		TextBox.say("...in five days.");
	}
	
	return {
		create: create,
		talk: talk
	};
}

/**
 * Register handles.
 */
({
	save: save,
	load: load,

	mapEnter: mapEnter,
	mapLeaveSouth: mapLeaveSouth,
	
	King: King(),
	StoreGuy: StoreGuy(),
	InnGuy: InnGuy(),
	Spinner: Spinner(),
	male_1: male_1(),
	Tunginobi: Tunginobi()
});